from .pipeline import ELTPipeline
from .extract import Extractor
from .transform import Transformer
from .load import Loader

__all__ = [
    "ELTPipeline",
    "Extractor",
    "Transformer",
    "Loader",
]



import requests
import psycopg2 
import pandas as pd 


class Extractor:
    @staticmethod
    def from_csv(file_path:str):
        ###Extract data from a CSV file
        return pd.read_csv(file_path)# converting to pandas df

    @staticmethod
    def from_api(url):
        ### Extract data from an API
        response = requests.get(url)
        response.raise_for_status()  # Raises an exception for bad responses
        return response.json()

    @staticmethod
    def from_db(database:str,query:str,user:str,password:str,host:str='localhost',port:str='5432'):
        ### Extract data from a PostgreSQL database
        db_connection =psycopg2.connect(
            dbname=database,
            user=user,
            password=password,
            host=host,
            port=port) 
        data=pd.read_sql_query(query,db_connection)##reading data from db using pandas
        db_connection.close()
        return data


class ELTPipeline:
    def __init__(self, extractor, transformer, loader):
        self.extractor = extractor
        self.transformer = transformer
        self.loader = loader

    def run(self, extract_method, extract_args,
                  transform_steps, load_method, load_args):
        # Step 1: Extract
        data = extract_method(*extract_args)

        # Step 2: Transform
        for step in transform_steps:
            data = step(data)

        # Step 3: Load
        load_method(data, *load_args)
        
        
        



import pandas as pd
import sqlalchemy
import json

class Loader:
    @staticmethod
    def to_csv(df: pd.DataFrame, file_path: str):
        try:
            df.to_csv(file_path, index=False)
            print(f"DataFrame successfully written to CSV at {file_path}")
        except Exception as e:
            print(f"Failed to write to CSV: {e}")
            raise

    @staticmethod
    def to_json(df: pd.DataFrame, file_path: str):
        try:
            df.to_json(file_path, orient='records', lines=True)
            print(f"DataFrame successfully written to JSON at {file_path}")
        except Exception as e:
            print(f"Failed to write to JSON: {e}")
            raise

    @staticmethod
    def to_db(df: pd.DataFrame, table_name: str, database: str, user: str, password: str,
              host: str = 'localhost', port: str = '5432', driver: str = 'postgresql'):
        try:
            engine = sqlalchemy.create_engine(
                f'{driver}://{user}:{password}@{host}:{port}/{database}'
            )
            df.to_sql(table_name, con=engine, if_exists='replace', index=False)
            print(f"DataFrame successfully written to DB table '{table_name}'")
        except Exception as e:
            print(f"Failed to write to database: {e}")
            raise



import requests ###Extracting data from API
import psycopg2 ##extracting data from database
import pandas as pd ##Extracting data from csv

class Transformer:
    @staticmethod
   # Remove duplicates
    def remove_duplicates(data:pd.DataFrame):
        return data.drop_duplicates()

        # Handle missing values (drop rows with missing values)
    @staticmethod
    def remove_missing(data:pd.DataFrame):
        return data.dropna()

        # Rename columns if provided
    @staticmethod
    def rename_columns(data:pd.DataFrame,columns:dict):
        return data.rename(columns=columns)

        # Convert date columns to datetime (if specified)
    @staticmethod
    def convert_datetime(data:pd.DataFrame,column:str):
        if column in data.columns:
            data[column] = pd.to_datetime(data[column], errors='coerce')

        return data

